==> org-lsst-ccs-image-utilities-3.1.1-sources.jar: org/lsst/ccs/utilities/readout/ReadOutParametersNew.java <==
package org.lsst.ccs.utilities.readout;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.utilities.ccd.CCDType;
import org.lsst.ccs.utilities.ccd.E2VCCDType;
import org.lsst.ccs.utilities.ccd.ITLCCDType;

/**
 *
 * @author tonyj
 */
public class ReadOutParametersNew extends ReadOutParameters {

    private static final Logger LOG = Logger.getLogger(ReadOutParametersNew.class.getName());
    private static final String UNDER_COLS = "UnderCols";
    private static final String PRE_COLS = "PreCols";
    private static final String READ_COLS = "ReadCols";
    private static final String POST_COLS = "PostCols";
    private static final String READ_COLS2 = "ReadCols2";
    private static final String OVER_COLS = "OverCols";
    private static final String PRE_ROWS = "PreRows";
    private static final String READ_ROWS = "ReadRows";
    private static final String POST_ROWS = "PostRows";
    private static final String OVER_ROWS = "OverRows";
    private static final String OP_FLAGS = "OpFlags";
    
    public static final String[] DEFAULT_NAMES = {
        UNDER_COLS, PRE_COLS, READ_COLS, POST_COLS, READ_COLS2, OVER_COLS, 
        PRE_ROWS, READ_ROWS, POST_ROWS, OVER_ROWS, OP_FLAGS}; 

    ReadOutParametersNew(int[] registers, ReadOutRegisterMap registerMap) {
        super(computeCCDType(registers, registerMap), registers, registerMap);
    }

    @Override
    public int getSerialReadPixels() {
        return getValue(UNDER_COLS) + getValue(PRE_COLS) + getValue(READ_COLS) + getValue(READ_COLS2) + getValue(OVER_COLS);
    }

    @Override
    public int getParallelReadPixels() {
        return getValue(PRE_ROWS) + getValue(READ_ROWS) + getValue(POST_ROWS) + getValue(OVER_ROWS);
    }

    @Override
    void validate() {
        boolean isValid = true;
        CCDType ccdType = getCCDType();

        int totalParallelPixels = ccdType.getCCDGeometryConstants().getSegmentParallelActiveSize();
        if (getValue(PRE_ROWS) + getValue(READ_ROWS) + getValue(OVER_ROWS) != totalParallelPixels) {
            LOG.log(Level.WARNING, "Parallel parameters are invalid: {0}", this);
            isValid = false;
        }
        int totalSerialPixels = ccdType.getCCDGeometryConstants().getSegmentSerialActiveSize() + ccdType.getCCDGeometryConstants().getSegmentSerialPrescanSize();
        if (getValue(UNDER_COLS) + getValue(PRE_COLS) + getValue(READ_COLS) + getValue(READ_COLS2) != totalSerialPixels) {
            LOG.log(Level.WARNING, "Serial parameters are invalid: {0}", this);
            isValid = false;
        }
        if (!isValid) {
            LOG.warning("The geometry values in the Fits Headers will be incorrect.");
        }
    }

    private static CCDType computeCCDType(int[] registers, ReadOutRegisterMap registerMap) {
        int opFlags = registers[registerMap.getIndex(OP_FLAGS)];
        if ((opFlags & 1) != 0) {
            return new ITLCCDType();
        } else if ((opFlags & 2) != 0) {
            return new E2VCCDType();
        } else {
            throw new RuntimeException(String.format("CCDType cannot be obtained from opFlags: %x", opFlags));
        }
    }

    @Override
    public int getTotalParallelSize() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public int getSerialPrescan() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public int getTotalSerialSize() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

}

==> org-lsst-ccs-image-utilities-3.1.21-20250206.164641-5-sources.jar: org/lsst/ccs/utilities/readout/ReadOutParametersNew.java <==
package org.lsst.ccs.utilities.readout;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.utilities.ccd.CCDType;
import org.lsst.ccs.utilities.ccd.E2VCCDType;
import org.lsst.ccs.utilities.ccd.ITLCCDType;

/**
 *
 * @author tonyj
 */
public class ReadOutParametersNew extends ReadOutParameters {

    private static final Logger LOG = Logger.getLogger(ReadOutParametersNew.class.getName());
    private static final String UNDER_COLS = "UnderCols";
    private static final String PRE_COLS = "PreCols";
    private static final String READ_COLS = "ReadCols";
    private static final String POST_COLS = "PostCols";
    private static final String READ_COLS2 = "ReadCols2";
    private static final String OVER_COLS = "OverCols";
    private static final String PRE_ROWS = "PreRows";
    private static final String READ_ROWS = "ReadRows";
    private static final String POST_ROWS = "PostRows";
    private static final String OVER_ROWS = "OverRows";
    private static final String OP_FLAGS = "OpFlags";
    
    public static final String[] DEFAULT_NAMES = {
        UNDER_COLS, PRE_COLS, READ_COLS, POST_COLS, READ_COLS2, OVER_COLS, 
        PRE_ROWS, READ_ROWS, POST_ROWS, OVER_ROWS, OP_FLAGS}; 

    ReadOutParametersNew(int[] registers, ReadOutRegisterMap registerMap) {
        super(computeCCDType(registers, registerMap), registers, registerMap);
    }

    @Override
    public int getSerialReadPixels() {
        return getValue(UNDER_COLS) + getValue(READ_COLS) + getValue(READ_COLS2) + getValue(OVER_COLS);
    }

    @Override
    public int getParallelReadPixels() {
        return getValue(READ_ROWS) + getValue(OVER_ROWS);
    }

    @Override
    void validate() {
        boolean isValid = true;
        CCDType ccdType = getCCDType();

        int totalParallelPixels = ccdType.getCCDGeometryConstants().getSegmentParallelActiveSize();
        if (getValue(PRE_ROWS) + getValue(READ_ROWS) + getValue(OVER_ROWS) != totalParallelPixels) {
            LOG.log(Level.WARNING, "Parallel parameters are invalid: {0}", this);
            isValid = false;
        }
        int totalSerialPixels = ccdType.getCCDGeometryConstants().getSegmentSerialActiveSize() + ccdType.getCCDGeometryConstants().getSegmentSerialPrescanSize();
        if (getValue(UNDER_COLS) + getValue(PRE_COLS) + getValue(READ_COLS) + getValue(READ_COLS2) != totalSerialPixels) {
            LOG.log(Level.WARNING, "Serial parameters are invalid: {0}", this);
            isValid = false;
        }
        if (!isValid) {
            LOG.warning("The geometry values in the Fits Headers will be incorrect.");
        }
    }

    private static CCDType computeCCDType(int[] registers, ReadOutRegisterMap registerMap) {
        int opFlags = registers[registerMap.getIndex(OP_FLAGS)];
        if ((opFlags & 1) != 0) {
            return new ITLCCDType();
        } else if ((opFlags & 2) != 0) {
            return new E2VCCDType();
        } else {
            throw new RuntimeException(String.format("CCDType cannot be obtained from opFlags: %x", opFlags));
        }
    }

    @Override
    public int getTotalParallelSize() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public int getSerialPrescan() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public int getTotalSerialSize() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

}
